# Generated by Django 3.2.17 on 2023-02-24 12:37
import logging

from django.db import migrations, models

logger = logging.getLogger(__name__)


def insert_treatment_ceiling_type(apps):
    Product = apps.get_model('product', 'Product')
    products = Product.objects.all().filter(
        validity_to__isnull=True,
        ceiling_type__isnull=True,
        ded_op_treatment__isnull=False,
        ded_ip_treatment__isnull=False,
        ded_treatment__isnull=False,
        max_treatment__isnull=False,
        max_ip_treatment__isnull=False,
        max_op_treatment__isnull=False
    )
    insert_values(products, "ceiling_type", "T")


def insert_policy_ceiling_type(apps):
    Product = apps.get_model('product', 'Product')
    products = Product.objects.all().filter(
        validity_to__isnull=True,
        ceiling_type__isnull=True,
        ded_op_policy__isnull=False,
        ded_ip_policy__isnull=False,
        ded_policy__isnull=False,
        max_policy__isnull=False,
        max_ip_policy__isnull=False,
        max_op_policy__isnull=False
    )
    insert_values(products, "ceiling_type", "P")


def insert_insuree_ceiling_type(apps):
    Product = apps.get_model('product', 'Product')
    products = Product.objects.all().filter(
        validity_to__isnull=True,
        ceiling_type__isnull=True
    )
    insert_values(products, "ceiling_type", "I")


def insert_values(list_of_objects: list, field: str, value: str):
    for single_object in list_of_objects:
        setattr(single_object, field, value)
        single_object.save()


def insert_ceiling_types(apps, schema_editor):
    """
    Insert ceiling types based on ded/max values.
    Default value is set to "Insuree"
    """
    insert_treatment_ceiling_type(apps)
    insert_policy_ceiling_type(apps)
    insert_insuree_ceiling_type(apps)


class Migration(migrations.Migration):

    dependencies = [
        ('product', '0005_add_ceiling_type_column'),
    ]

    operations = [
        migrations.RunPython(insert_ceiling_types, migrations.RunPython.noop),
    ]
